searchNodes=[{"doc":"Path Express provides nil-safe list navigation for use with Kernel.get_in/2 plus its own get_in/2 wrapper with some shortcuts.","ref":"PathExpress.html","title":"PathExpress","type":"module"},{"doc":"Returns a function that accesses all the elements in a list. The returned function is typically passed as an accessor to get_in/2 and Kernel.get_in/2. Examples iex&gt; list = [%{name: &quot;john&quot;}, %{name: &quot;mary&quot;}] iex&gt; get_in(list, [PathExpress.all(), :name]) [&quot;john&quot;, &quot;mary&quot;] An empty list is a shortcut to all/0 when PathExpress.get_in/2 used iex&gt; data = %{items: [%{qty: 1}, %{qty: 2}]} iex&gt; PathExpress.get_in(data, [:items, [], :qty]) [1, 2] nil or a non-list is traversed by returning empty list: iex&gt; get_in(nil, [PathExpress.all()]) [] iex&gt; get_in(%{}, [PathExpress.all()]) []","ref":"PathExpress.html#all/0","title":"PathExpress.all/0","type":"function"},{"doc":"Returns a function that accesses the element at index (zero based) of a list. The returned function is typically passed as an accessor to get_in/2 and Kernel.get_in/2. Examples # iex&gt; list = [%{name: &quot;john&quot;}, %{name: &quot;mary&quot;}] iex&gt; get_in(list, [PathExpress.at(1), :name]) &quot;mary&quot; iex&gt; get_in(list, [PathExpress.at(-1), :name]) &quot;mary&quot; nil or a non-list is traversed by returning nil: iex&gt; get_in(nil, [PathExpress.at(0)]) nil iex&gt; get_in(%{}, [PathExpress.at(1)]) nil","ref":"PathExpress.html#at/1","title":"PathExpress.at/1","type":"function"},{"doc":"Returns a function that accesses all elements of a list that match the provided predicate. The returned function is typically passed as an accessor to get_in/2 and Kernel.get_in/2. Examples iex&gt; list = [%{name: &quot;john&quot;, salary: 10}, %{name: &quot;francine&quot;, salary: 30}] iex&gt; get_in(list, [PathExpress.filter(&amp;(&amp;1.salary &gt; 20)), :name]) [&quot;francine&quot;] When no match is found, an empty list is returned: iex&gt; list = [%{name: &quot;john&quot;, salary: 10}, %{name: &quot;francine&quot;, salary: 30}] iex&gt; get_in(list, [PathExpress.filter(&amp;(&amp;1.salary &gt;= 50)), :name]) [] nil or a non-list is traversed by returning an empty list, as if no match is found: iex&gt; get_in(nil, [PathExpress.filter(&amp;(&amp;1.salary &gt;= 50)), :name]) [] iex&gt; get_in(%{}, [PathExpress.filter(fn a -&gt; a == 10 end)]) [] An error is raised if the predicate is not a function or is of the incorrect arity: iex&gt; get_in([], [PathExpress.filter(5)]) ** (FunctionClauseError) no function clause matching in PathExpress.filter/1","ref":"PathExpress.html#filter/1","title":"PathExpress.filter/1","type":"function"},{"doc":"Allows pulling multiple paths out of a data tree into a map. iex&gt; data = %{a: %{b: &quot;pick me&quot;}, c: [%{d: &quot;pick me too&quot;}]} iex&gt; mappings = %{me: [:a, :b], me2: [:c, 0, :d]} iex&gt; PathExpress.get_all_in(data, mappings) %{me: &quot;pick me&quot;, me2: &quot;pick me too&quot;}","ref":"PathExpress.html#get_all_in/2","title":"PathExpress.get_all_in/2","type":"function"},{"doc":"Gets a value from a nested structure. Uses a combination of functions from the Access module and PathExpress to traverse the structures in a nil-safe way according to the given keys, unless the key is a function, which is detailed in a later section. Examples iex&gt; users = %{&quot;john&quot; =&gt; %{age: 27}, &quot;meg&quot; =&gt; %{age: 23}} iex&gt; get_in(users, [&quot;john&quot;, :age]) 27 In case any of the keys returns nil, nil will be returned: iex&gt; users = %{&quot;john&quot; =&gt; %{age: 27}, &quot;meg&quot; =&gt; %{age: 23}} iex&gt; get_in(users, [&quot;unknown&quot;, :age]) nil iex&gt; get_in(users, [&quot;unknown&quot;, PathExpress.at(0)]) nil Shortcuts Integers in keys will be treated as PathExpress.at/1 and an empty list will be treated as PathExpress.all/ iex&gt; alias PathExpress, as: PE iex&gt; data = %{&quot;users&quot; =&gt; [%{name: &quot;john&quot;, age: 27}, %{name: &quot;meg&quot;, age: 23}]} iex&gt; PE.get_in(data, [&quot;users&quot;, 0, :name]) &quot;john&quot; iex&gt; PE.get_in(data, [&quot;users&quot;, [], :age]) [27, 23] The examples above are nil-safe: iex&gt; alias PathExpress, as: PE iex&gt; data = %{} iex&gt; PE.get_in(data, [&quot;users&quot;, 0, :name]) nil iex&gt; PE.get_in(data, [&quot;users&quot;, [], :age]) [] Functions as keys See documentation for Kernel.get_in/2","ref":"PathExpress.html#get_in/2","title":"PathExpress.get_in/2","type":"function"},{"doc":"Returns a function that maps all the elements in a list through your function ref. The returned function is typically passed as an accessor to get_in/2 and Kernel.get_in/2. Examples iex&gt; alias PathExpress, as: PE iex&gt; data = %{names: [&quot;john&quot;, &quot;mary&quot;]} iex&gt; get_in(data, [:names, PE.map_all(&amp;String.upcase/1)]) [&quot;JOHN&quot;, &quot;MARY&quot;] A list containing the function ref is a shortcut to map_all/1 when PathExpress.get_in/2 used iex&gt; data = %{quantities: [1, 2]} iex&gt; PathExpress.get_in(data, [:quantities, [&amp;(&amp;1 + 1)]]) [2, 3] nil or a non-list is traversed by returning empty list: iex&gt; get_in(nil, [PathExpress.map_all(&amp;String.upcase/1)]) [] iex&gt; get_in(%{}, [PathExpress.map_all(&amp;String.upcase/1)]) []","ref":"PathExpress.html#map_all/1","title":"PathExpress.map_all/1","type":"function"},{"doc":"PathExpress PathExpress is a shortened form of &quot;path expression&quot; also intended to bring brevity to mind. I first encountered the concept of path expressions as a way to navigate object-oriented databases. You can also imagine a similarity with XPath, used to navigate xml documents.","ref":"readme.html","title":"PathExpress","type":"extras"},{"doc":"I need to pull values out of json documents from untrusted third parties at the edges of my system. There&#39;s multiple incoming json structures from different sources that I map to a common Ecto struct for core business logic processing. The initial step is to navigate paths to fields in the json and save in a map of names known to my changeset function. Kernel.get_in/2 is nil-safe if you are navigating purely a path of maps embedded in maps, however once you need to navigate steps that deal with lists, then you open the chance of nil exceptions when keys are missing or are nil instead of an empty list. For these reasons I developed PathExpress.","ref":"readme.html#rationale","title":"PathExpress - Rationale","type":"extras"},{"doc":"You can use PathExpress as a replacement for the Access module functions that operate on lists in any existing get_in/2 calls and gain the same nil safety for lists as it has for maps. Example Before iex&gt; %{&quot;items&quot; =&gt; nil} |&gt; get_in([&quot;items&quot;, Access.at(0)]) ** (RuntimeError) Access.at/1 expected a list, got: nil (elixir) lib/access.ex:663: Access.at/4 After iex&gt; %{&quot;items&quot; =&gt; nil} |&gt; get_in([&quot;items&quot;, PathExpress.at(0)]) nil","ref":"readme.html#compatible-with-kernel-get_in-2","title":"PathExpress - Compatible with Kernel.get_in/2","type":"extras"},{"doc":"If you do this often, you can gain some brevity by letting PathExpress wrap the calls to get_in/2 for you. In those cases, there are shortcuts for specifying the path elements involving lists. An integer retrieves that element of a list, like PathExpress.at/1 and an empty list acts as PathExpress.all/0. The example above could be expressed as: iex&gt; alias PathExpress, as: PE iex&gt; %{&quot;items&quot; =&gt; nil} |&gt; PE.get_in([&quot;items&quot;, 0]) nil In the rare cases (in my experience) that you might want to deal with map keys that are integers or an empty list, then Access.key/1 has you covered.","ref":"readme.html#the-express-part-of-the-name","title":"PathExpress - The &quot;Express&quot; part of the name","type":"extras"},{"doc":"Addpath_express to your list of dependencies in mix.exs: def deps do [ {:path_express, &quot;~&gt; 0.2.0&quot;} ] end Docs at https://hexdocs.pm/path_express.","ref":"readme.html#installation","title":"PathExpress - Installation","type":"extras"},{"doc":"get_in/3 that takes a default value when path cannot be navigated get_in!/2 which assertively raises if any path elements not found (key or list style) fetch_in/2 which returns {:ok, value} or :error better CI handling of Tuple paths? handling of put_in, update_in pop_in, get_and_update_in (discussing plans)","ref":"readme.html#todo","title":"PathExpress - TODO","type":"extras"}]